# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/status/main.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..status import scanner_pb2 as blickfeld_dot_status_dot_scanner__pb2
from ..status import temperature_pb2 as blickfeld_dot_status_dot_temperature__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/status/main.proto',
  package='blickfeld.protocol',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x1b\x62lickfeld/status/main.proto\x12\x12\x62lickfeld.protocol\x1a\x1e\x62lickfeld/status/scanner.proto\x1a\"blickfeld/status/temperature.proto\"{\n\x06Status\x12\x33\n\x07scanner\x18\x01 \x01(\x0b\x32\".blickfeld.protocol.status.Scanner\x12<\n\x0ctemperatures\x18\x02 \x03(\x0b\x32&.blickfeld.protocol.status.Temperature')
  ,
  dependencies=[blickfeld_dot_status_dot_scanner__pb2.DESCRIPTOR,blickfeld_dot_status_dot_temperature__pb2.DESCRIPTOR,])




_STATUS = _descriptor.Descriptor(
  name='Status',
  full_name='blickfeld.protocol.Status',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='scanner', full_name='blickfeld.protocol.Status.scanner', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='temperatures', full_name='blickfeld.protocol.Status.temperatures', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=119,
  serialized_end=242,
)

_STATUS.fields_by_name['scanner'].message_type = blickfeld_dot_status_dot_scanner__pb2._SCANNER
_STATUS.fields_by_name['temperatures'].message_type = blickfeld_dot_status_dot_temperature__pb2._TEMPERATURE
DESCRIPTOR.message_types_by_name['Status'] = _STATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Status = _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), dict(
  DESCRIPTOR = _STATUS,
  __module__ = 'blickfeld.status.main_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.Status)
  ))
_sym_db.RegisterMessage(Status)


# @@protoc_insertion_point(module_scope)
